/*
	"  *************************************************\n"\
	"  *          Generation d'Arbre Arbore            *\n"\
	"  *       fait par Jean-Charles BONTEMPS          *\n"\
	"  *                Licence GPL                    *\n"\
	"  *   http://poulphunter1.free.fr/ArbreArbore     *\n"\
	"  *           poulphunter@yahoo.fr                *\n"\
	"  *************************************************\n\n"\
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h> /*log() */

#include "fonts.h"  //str_width, str_height
#include "cooccurrences.h" //macro distance, cooccurrences, distances
#include "read_write.h" //freq_buffer, text_buffer, point_texte
#include "arbredico.h" //Arbre_Dico_Init, Arbre_Dico_ADD, Arbre_Dico_FOUND
#include "SVG.h"
#include "Phylo.h"

PhyloNode *PhyloNode_Create()
{
	PhyloNode *t=malloc(sizeof(PhyloNode));
	if (t==0)
		return 0;
	t->Fils0=0;
	t->Fils1=0;
	t->weight=0;
	t->angle=0;
	t->name[0]=0;
	t->size=10;
	//t->r=0;
	//t->g=0;
	//t->b=0;
	t->x=50000;
	t->y=50000;
	t->refnumber=0;
	return t;
}
PhyloTree *PhyloTree_Create()
{
	PhyloTree *t=malloc(sizeof(PhyloTree));
	t->angle=0;
	t->Newick[0]=0;
	//t->r=200;
	//t->g=210;
	//t->b=255;
	t->xmin=100000;
	t->ymin=100000;
	t->xmax=-1;
	t->ymax=-1;
	t->root=0;
return t;
}

int sortLeaves(point_texte* pt,PhyloNode *t,int sofar,int tre_angle)
{
	int isleaf=0;
	int res=sofar;
	if (t->Fils0!=0){isleaf=1;res=sortLeaves(pt,t->Fils0,res,tre_angle);}
	if (t->Fils1!=0){isleaf=1;res=sortLeaves(pt,t->Fils1,res,tre_angle);}
	if (isleaf==0)
	{
		#define pi 3.14159265358979323846
		t->angle=(sofar*2*pi)/(pt->nbgarde+2+tre_angle);
		t->minangle=t->angle;
		t->maxangle=t->angle;
		t->weight=1;
		res=sofar+1;
	}
	else
	{   
		if (t->Fils1!=0)
		{
			t->minangle=min(t->Fils0->minangle,t->Fils1->minangle);
			t->maxangle=max(t->Fils0->maxangle,t->Fils1->maxangle);
			t->angle=(t->minangle+t->maxangle)/2;
		} else {
			t->angle=(t->minangle+t->maxangle)/2;
		}	
	}
	return res;
}

void RectN_add(RectN *rec,int x,int y,int w,int h,int re,int g,int b,int fontsize,int freq,char *st,int l0x,int l0y,int l1x,int l1y)
{ //insert between first and 2nd RectN in list (like that we don't have to give back the pointer of new list)
	if (rec==0)
		return;
	RectN *r=malloc(sizeof(RectN));
	r->next=rec->next;
	r->x=x;
	r->y=y;
	r->width=w;
	r->height=h;
	r->x1=x+w;
	r->y1=y+h;
	r->vx=0;
	r->vy=0;
	r->r=re;
	r->g=g;
	r->b=b;
	r->fontsize=fontsize;
	r->freq=freq;
	r->l0x=l0x;
	r->l0y=l0y;
	r->l1x=l1x;
	r->l1y=l1y;
	strncpy(r->str,st,256);
	rec->next=r;
}

void computeDrawing(PhyloNode *t,double x,double y,PhyloTree *tre)
{
	if (t->Fils0!=0)
	{
		t->Fils0->x=floor(x+50*cos(t->Fils0->angle));
		t->Fils0->y=floor(y+50*sin(t->Fils0->angle));
		if (t==tre->root) {t->Fils0->x=t->x;t->Fils0->y=t->y;}
		if (t->Fils0->x<tre->xmin) tre->xmin=t->Fils0->x;
		if (t->Fils0->y<tre->ymin) tre->ymin=t->Fils0->y;
		if (t->Fils0->x>tre->xmax) tre->xmax=t->Fils0->x;
		if (t->Fils0->y>tre->ymax) tre->ymax=t->Fils0->y;
		computeDrawing(t->Fils0,t->Fils0->x,t->Fils0->y,tre);
		//printf("tfils %f %f\n",t->Fils0->x,t->Fils0->y);
		
	}
	if (t->Fils1!=0)
	{
		t->Fils1->x=floor(x+50*cos(t->Fils1->angle));
		t->Fils1->y=floor(y+50*sin(t->Fils1->angle));
		if (t==tre->root) {t->Fils1->x=t->x;t->Fils1->y=t->y;}
		if (t->Fils1->x<tre->xmin) tre->xmin=t->Fils1->x;
		if (t->Fils1->y<tre->ymin) tre->ymin=t->Fils1->y;
		if (t->Fils1->x>tre->xmax) tre->xmax=t->Fils1->x;
		if (t->Fils1->y>tre->ymax) tre->ymax=t->Fils1->y;
		computeDrawing(t->Fils1,t->Fils1->x,t->Fils1->y,tre);
		//printf("tfils %f %f\n",t->Fils0->x,t->Fils0->y);
	}
}

void draw(point_texte* pt,text_buffer *imageSVG,PhyloNode *t,int x,int y,int xx,int yy,PhyloTree *tre,double frmax,double frmin,RectN *rec)
{
	double a=(x-2*xx)/(tre->xmax-tre->xmin);
	double b=(y-2*yy)/(tre->ymax-tre->ymin);
	double newx,newy,level,level1,xm,ym,lx0,ly0,lx1,ly1;
	int h=0,w=0,angle;
	int k;//color tre->r,tre->g,tre->b
	if (t->Fils0!=0)
	{
		//draw the edge to the currently treated son:
		lx0=floor((t->x-tre->xmin)*a)+xx;
		ly0=floor((t->y-tre->ymin)*b)+yy;
		lx1=floor((t->Fils0->x-tre->xmin)*a)+xx;
		ly1=floor((t->Fils0->y-tre->ymin)*b)+yy;
		if (t->Fils0->name[0]!=0 && t->Fils0->name[0]!='(')
		{
			//si le nom n'est pas vide:
			k=t->Fils0->refnumber-1;
			level=floor(55+200*((log2((double)freq(k))-log2(frmin))/(log2(frmax)-log2(frmin))));
			//en fonction du nombre de mot dfinit la taille.
			level1=floor((level*((int)ceil((200-min(((double)pt->nbgarde),200.0))*15.0/200.0)))/256)+10;
			//level1=((int)ceil((200-min(((double)pt->nbgarde),200.0))*15.0/200.0))+5+floor((level*5)/256);
			h=ceil(str_height(level1)/2.0);
			w=str_width(t->Fils0->name,1,level1);
			ym=h;
			xm=w+10;
			newx=(t->Fils0->x-tre->xmin)*a-(xm*sin(t->Fils0->angle/2))+xx+5;
            newy=(t->Fils0->y-tre->ymin)*b-(ym)/2*sin(t->Fils0->angle+pi)+5+yy;
			angle=0;
			if (t->Fils0->angle>=pi/2 && t->Fils0->angle<pi)
				RectN_add(rec,lx1-(w+10),ly1,w+10,h*2,level,(level/2),(255-level),level1,freq(k),t->Fils0->name,lx1,ly1,lx1,ly1);
			if (t->Fils0->angle>=0 && t->Fils0->angle<pi/2)
				RectN_add(rec,lx1,ly1,w+10,h*2,level,(level/2),(255-level),level1,freq(k),t->Fils0->name,lx1,ly1,lx1,ly1);
			if (t->Fils0->angle>=pi*3/2 && t->Fils0->angle<=pi*2)
				RectN_add(rec,lx1,ly1-h*2,w+10,h*2,level,(level/2),(255-level),level1,freq(k),t->Fils0->name,lx1,ly1,lx1,ly1);
			if (t->Fils0->angle>=pi && t->Fils0->angle<pi*3/2)
				RectN_add(rec,lx1-(w+10),ly1-(h*2),w+10,h*2,level,(level/2),(255-level),level1,freq(k),t->Fils0->name,lx1,ly1,lx1,ly1);
		}
		imageSVG->size+=SVG_Line(&imageSVG->buffer[imageSVG->size],(int)lx0,(int)ly0,(int)lx1,(int)ly1);
		draw(pt,imageSVG,t->Fils0,x,y,xx,yy,tre,frmax,frmin,rec);
		
	}
	
	if (t->Fils1!=0 && t->Fils1!=t->Fils0)
	{
		//draw the edge to the currently treated son:
		lx0=floor((t->x-tre->xmin)*a)+xx;
		ly0=floor((t->y-tre->ymin)*b)+yy;
		lx1=floor((t->Fils1->x-tre->xmin)*a)+xx;
		ly1=floor((t->Fils1->y-tre->ymin)*b)+yy;     
		if (t->Fils1->name[0]!=0 && t->Fils1->name[0]!='(')
		{
			//si le nom n'est pas vide:
			k=t->Fils1->refnumber-1;
			level=floor(55+200*((log2((double)freq(k))-log2(frmin))/(log2(frmax)-log2(frmin))));
			//en fonction du nombre de mot dfinit la taille.
			level1=floor((level*((int)ceil((200-min(((double)pt->nbgarde),200.0))*10.0/200.0)))/256)+10;
			h=ceil(str_height(level1)/2.0);
			w=str_width(t->Fils1->name,1,level1);
			ym=h;
			xm=w+10;
			newx=(t->Fils1->x-tre->xmin)*a-(xm*sin(t->Fils1->angle/2))+xx+5;
            newy=(t->Fils1->y-tre->ymin)*b-(ym)/2*sin(t->Fils1->angle+pi)+5+yy;
			angle=0;
			if (t->Fils1->angle>=pi/2 && t->Fils1->angle<pi)
				RectN_add(rec,lx1-(w+10),ly1,w+10,h*2,level,(level/2),(255-level),level1,freq(k),t->Fils1->name,lx1,ly1,lx1,ly1);
			if (t->Fils1->angle>=0 && t->Fils1->angle<pi/2)
				RectN_add(rec,lx1,ly1,w+10,h*2,level,(level/2),(255-level),level1,freq(k),t->Fils1->name,lx1,ly1,lx1,ly1);
			if (t->Fils1->angle>=pi*3/2 && t->Fils1->angle<=pi*2)
				RectN_add(rec,lx1,ly1-h*2,w+10,h*2,level,(level/2),(255-level),level1,freq(k),t->Fils1->name,lx1,ly1,lx1,ly1);
			if (t->Fils1->angle>=pi && t->Fils1->angle<pi*3/2)
				RectN_add(rec,lx1-(w+10),ly1-(h*2),w+10,h*2,level,(level/2),(255-level),level1,freq(k),t->Fils1->name,lx1,ly1,lx1,ly1);
		} 
		imageSVG->size+=SVG_Line(&imageSVG->buffer[imageSVG->size],(int)lx0,(int)ly0,(int)lx1,(int)ly1);
		draw(pt,imageSVG,t->Fils1,x,y,xx,yy,tre,frmax,frmin,rec);
	}
}

#define col_limite 1
void draw_end(RectN *rec,text_buffer *imageSVG,int vector_percent){
	if (rec==0)
		return;
	int vx=ceil((double)vector_percent*rec->vx/100.0);
	int vy=ceil((double)vector_percent*rec->vy/100.0);
	
	imageSVG->size+=SVG_Text(&imageSVG->buffer[imageSVG->size],rec->x+5+vx,rec->y+(rec->height*4/5)+vy,rec->r,rec->g,rec->b,rec->str,rec->freq,rec->fontsize);
	// imageSVG->size+=SVG_Rect(&imageSVG->buffer[imageSVG->size],rec->x+vx+col_limite,rec->y+vy+col_limite,rec->width-(2*col_limite),rec->height-(2*col_limite));
	// imageSVG->size+=SVG_Line_Point(&imageSVG->buffer[imageSVG->size],rec->l0x,rec->l0y,rec->l1x+vx,rec->l1y+vy);
	draw_end(rec->next,imageSVG,vector_percent);
}

// ((r2->x<=r1->x1 && r2->x>=r1->x && ((r2->y<=r1->y1 && r2->y>=r1->y) || (r2->y1<=r1->y1 && r2->y1>=r1->y))) ||
// (r2->x1<=r1->x1 && r2->x1>=r1->x && ((r2->y<=r1->y1 && r2->y>=r1->y) || (r2->y1<=r1->y1 && r2->y1>=r1->y))) ||
// (r1->x<=r2->x1 && r1->x>=r2->x && ((r1->y<=r2->y1 && r1->y>=r2->y) || (r1->y1<=r2->y1 && r1->y2>=r1->y))) ||
// (r1->x1<=r2->x1 && r1->x1>=r2->x && ((r1->y<=r2->y1 && r1->y>=r2->y) || (r1->y1<=r2->y1 && r1->y2>=r1->y))))

int inclu(RectN *r1,RectN *r2)
{
	if (r1->x<r2->x && r1->y<r2->y && r1->x1>r2->x1 && r1->y1>r2->y1)
		return 1;
	return 0;
}

int chevauche(RectN *r1,RectN *r2)
{
	if (r1->x<r2->x && r1->y<r2->y && r1->x1>r2->x && r1->y1>r2->y && r1->x1>r2->x1)
		return 1;
	if (r1->x<r2->x && r1->y<r2->y && r1->x1>r2->x && r1->y1>r2->y && r1->y1>r2->y1)
		return 1;
	if (r1->x1>r2->x1 && r1->y1>r2->y1 && r1->x<r2->x1 && r1->y<r2->y1 && r1->x<r2->x)
		return 1;
	if (r1->x1>r2->x1 && r1->y1>r2->y1 && r1->x<r2->x1 && r1->y<r2->y1 && r1->y<r2->y)
		return 1;
	return 0;
}

int collision(RectN *r1,RectN *r2)
{
	if (((r2->x<=r1->x1 && r2->x>=r1->x && ((r2->y<=r1->y1 && r2->y>=r1->y) || (r2->y1<=r1->y1 && r2->y1>=r1->y))) ||
		(r2->x1<=r1->x1 && r2->x1>=r1->x && ((r2->y<=r1->y1 && r2->y>=r1->y) || (r2->y1<=r1->y1 && r2->y1>=r1->y))) ||
		(r1->x<=r2->x1 && r1->x>=r2->x && ((r1->y<=r2->y1 && r1->y>=r2->y) || (r1->y1<=r2->y1 && r1->y1>=r2->y))) ||
		(r1->x1<=r2->x1 && r1->x1>=r2->x && ((r1->y<=r2->y1 && r1->y>=r2->y) || (r1->y1<=r2->y1 && r1->y1>=r2->y)))))
		return 1;
	return 0;
}

int compute_vector(RectN *r1){
#define col_lim 1

#define add_col r1->x+=col_limite;r1->x1-=(2*col_limite);r1->y+=col_limite;r1->y1-=(2*col_limite);r2->x+=col_limite;r2->x1-=(2*col_limite);r2->y+=col_limite;r2->y1-=(2*col_limite);
#define del_col r1->x-=col_limite;r1->x1+=(2*col_limite);r1->y-=col_limite;r1->y1+=(2*col_limite);r2->x-=col_limite;r2->x1+=(2*col_limite);r2->y-=col_limite;r2->y1+=(2*col_limite);

#define add_vector add_col;r1->x+=r1->vx;r1->x1+=r1->vx;r1->y+=r1->vy;r1->y1+=r1->vy;r2->x+=r2->vx;r2->x1+=r2->vx;r2->y+=r2->vy;r2->y1+=r2->vy;vx=0;vy=0;
#define del_vector del_col;r1->x-=r1->vx;r1->x1-=r1->vx;r1->y-=r1->vy;r1->y1-=r1->vy;r2->x-=r2->vx;r2->x1-=r2->vx;r2->y-=r2->vy;r2->y1-=r2->vy;
#define add 2
#define vx1 ((vx)<(0)?((vx)>(-1)?(-1):(vx)):((vx)<(1)?(1):(vx)))
#define vy1 ((vy)<(0)?((vy)>(-1)?(-1):(vy)):((vy)<(1)?(1):(vy)))
#define vx2 ((vx)<(0)?((vx)>(-col_lim)?(-1):(vx1)):((vx)<(col_lim)?(1):(vx1)))
#define vy2 ((vy)<(0)?((vy)>(-col_lim)?(-1):(vy1)):((vy)<(col_lim)?(1):(vy1)))

#define addv1 r1->vx+=vx2;r1->vy+=vy2;r2->vx-=vx2;r2->vy-=vy2;

#define cont1 del_vector;addv1;r2=r2->next;continue;
#define cont del_vector;r2=r2->next;continue;

	int vx,vy,ret=0;
	if (r1==0)
		return 0;
	RectN *r2=r1->next;
	while (r2!=NULL)
	{
		add_vector
		if (chevauche(r1,r2) || chevauche(r2,r1))
		{	ret=1;
			//printf("chevauche:(%s,%s)\n",r1->str,r2->str);
			if (r1->x<r2->x && r1->y<r2->y && r1->x1>r2->x && r1->y1>r2->y && r1->x1>r2->x1)
			{
				vy-=ceil((r2->y-r1->y)/2)+add;cont1
			}
			if (r1->x<r2->x && r1->y<r2->y && r1->x1>r2->x && r1->y1>r2->y && r1->y1>r2->y1)
			{
				vx-=ceil((r2->x-r1->x)/2)+add;cont1
			}
			if (r1->x1>r2->x1 && r1->y1>r2->y1 && r1->x<r2->x1 && r1->y<r2->y1 && r1->x<r2->x)
			{
				vy+=ceil((r2->y1-r1->y)/2)+add;cont1
			}
			if (r1->x1>r2->x1 && r1->y1>r2->y1 && r1->x<r2->x1 && r1->y<r2->y1 && r1->y<r2->y)
			{
				vx+=ceil((r2->x1-r1->x)/2)+add;cont1
			}
			if (r2->x<r1->x && r2->y<r1->y && r2->x1>r1->x && r2->y1>r1->y && r2->x1>r1->x1)
			{
				vy+=ceil((r1->y-r2->y)/2)+add;cont1
			}
			if (r2->x<r1->x && r2->y<r1->y && r2->x1>r1->x && r2->y1>r1->y && r2->y1>r1->y1)
			{
				vx+=ceil((r1->x-r2->x)/2)+add;cont1
			}
			if (r2->x1>r1->x1 && r2->y1>r1->y1 && r2->x<r1->x1 && r2->y<r1->y1 && r2->x<r1->x)
			{
				vy-=ceil((r1->y1-r2->y)/2)+add;cont1
			}
			if (r2->x1>r1->x1 && r2->y1>r1->y1 && r2->x<r1->x1 && r2->y<r1->y1 && r2->y<r1->y)
			{
				vx-=ceil((r1->x1-r2->x)/2)+add;cont1
			}
			
		} else {
			if (collision(r1,r2) || collision(r2,r1))
			{	ret=1;//printf("collision:(%s,%s)\n",r1->str,r2->str);
				if (r1->x<r2->x && r1->y<r2->y && r1->x1>r2->x && r1->y1>r2->y)
				{
					//dplace sur l'axe des x ou y selon l'eloignement que sa fait par rapport  la position d'origine
					if (abs(r1->vx-(ceil((r1->x1-r2->x)/2)+add))<abs(r1->vx))//si distance future de vx < vx actuel
					{
						vx-=ceil((r1->x1-r2->x)/2)+add;cont1
					}
					if (abs(r1->vy-ceil((r1->y1-r2->y)/2)+add)<abs(r1->vy))//si distance future de vy < vy actuel
					{
						vy-=ceil((r2->y-r1->y)/2)+add;cont1
					}
					//printf("%d  %d\n",(r1->y1-r2->y)/2+add,(r1->x1-r2->x)/2+add);
					if (ceil((r1->y1-r2->y)/2)+add<=ceil((r1->x1-r2->x)/2)+add) //sinon on prend le plus court
					{
						vy-=ceil((r1->y1-r2->y)/2)+add;cont1
					} else {
						vx-=ceil((r1->x1-r2->x)/2)+add;cont1
					}
				}
				if (r2->x<r1->x && r2->y<r1->y && r2->x1>r1->x && r2->y1>r1->y)
				{
					//dplace sur l'axe des x ou y selon l'eloignement que sa fait par rapport  la position d'origine
					if (abs(r2->vx+(ceil((r2->x1-r1->x)/2)+add))<abs(r2->vx))//si distance future de vx < vx actuel
					{
						vx+=ceil((r2->x1-r1->x)/2)+add;cont1
					}
					if (abs(r2->vy+ceil((r2->y1-r1->y)/2)+add)<abs(r2->vy))//si distance future de vy < vy actuel
					{
						vy+=ceil((r1->y-r2->y)/2)+add;cont1
					}
					//printf("%d  %d\n",(r2->y1-r1->y)/2+add,(r2->x1-r1->x)/2+add);
					if (ceil((r2->y1-r1->y)/2)+add<=ceil((r2->x1-r1->x)/2)+add) //sinon on prend le plus court
					{
						vy+=ceil((r2->y1-r1->y)/2)+add;cont1
					} else {
						vx+=ceil((r2->x1-r1->x)/2)+add;cont1
					}
				}
				
				if (r1->x<r2->x1  && r1->y<r2->y && r1->x1>r2->x1 && r1->y1>r2->y)
				{
					//dplace sur l'axe des x ou y selon l'eloignement que sa fait par rapport  la position d'origine
					if (abs(r1->vx+(ceil((r2->x1-r1->x)/2)+add))<abs(r1->vx))//si distance future de vx < vx actuel
					{
						vx+=ceil((r2->x1-r1->x)/2)+add;cont1
					}
					if (abs(r1->vy-ceil((r1->y1-r2->y)/2)+add)<abs(r1->vy))//si distance future de vy < vy actuel
					{
						vy-=ceil((r2->y-r1->y)/2)+add;cont1
					}
					//printf("%d  %d\n",(r1->y1-r2->y)/2+add,(r1->x1-r2->x)/2+add);
					if (ceil((r1->y1-r2->y)/2)+add<=ceil((r2->x1-r1->x)/2)+add) //sinon on prend le plus court
					{
						vy-=ceil((r1->y1-r2->y)/2)+add;cont1
					} else {
						vx+=ceil((r2->x1-r1->x)/2)+add;cont1
					}
				}
				if (r2->x<r1->x1  && r2->y<r1->y && r2->x1>r1->x1 && r2->y1>r1->y)
				{
					//dplace sur l'axe des x ou y selon l'eloignement que sa fait par rapport  la position d'origine
					if (abs(r2->vx-(ceil((r1->x1-r2->x)/2)+add))<abs(r2->vx))//si distance future de vx < vx actuel
					{
						vx-=ceil((r1->x1-r2->x)/2)+add;cont1
					}
					if (abs(r2->vy+ceil((r2->y1-r1->y)/2)+add)<abs(r2->vy))//si distance future de vy < vy actuel
					{
						vy+=ceil((r1->y-r2->y)/2)+add;cont1
					}
					//printf("%d  %d\n",(r2->y1-r1->y)/2+add,(r2->x1-r1->x)/2+add);
					if (ceil((r2->y1-r1->y)/2)+add<=ceil((r1->x1-r2->x)/2)+add) //sinon on prend le plus court
					{
						vy+=ceil((r2->y1-r1->y)/2)+add;cont1
					} else {
						vx-=ceil((r1->x1-r2->x)/2)+add;cont1
					}
				}
				if (r1->x==r2->x)
				{
					vx+=1;cont1
				}
				if (r1->y==r2->y)
				{
					vy+=1;cont1
				}
				if (r1->x1==r2->x1)
				{
					vx+=1;cont1
				}
				if (r1->y1==r2->y1)
				{
					vy+=1;cont1
				}
			}
		}
		cont
	}
	if (compute_vector(r1->next)==1)
		return 1;
	else
		return ret;
}

void RectN_free(RectN *rec)
{
	if (rec==0)
		return;
	RectN_free(rec->next);
	free(rec);
}

int find_max(RectN *r1){
	RectN *r=r1;
	int max=0;
	while (r!=0)
	{
		if (abs(r->vx)>max)
			max=abs(r->vx);
		if (abs(r->vy)>max)
			max=abs(r->vy);
		r=r->next;
	}
	return max;
}

void red_vector(RectN *r1){
	//int max=find_max(r1)/2;
	RectN *r=r1;
	while (r!=0)
	{
		// if (abs(r->vx)>max/2 || abs(r->vy)>max/2)
		// {
			if (r1->vx>0)
				r1->vx-=2;
			if (r1->vx<0)
				r1->vx+=2;
			if (r1->vy>0)
				r1->vy-=2;
			if (r1->vy<0)
				r1->vy+=2;
		// }
		r=r->next;
	}
}

void pre_draw(point_texte* pt,text_buffer *imageSVG,PhyloNode *t,int x,int y,int xx,int yy,PhyloTree *tre)
{
	int i,j;
	//Calcul qu'une fois la frequence max et min
	double frmax=freq(0)*1.0;
	double frmin=freq(0)*1.0;
	for (i=0;i<pt->nbgarde;i++)
	{
		if (freq(i)>frmax) {frmax=freq(i);}
		if (freq(i)<frmin) {frmin=freq(i);}
	}
	RectN *rec=malloc(sizeof(RectN));
	rec->x=0;rec->y=0;rec->width=0;rec->height=0;rec->x1=0;rec->y1=0;rec->vx=0;rec->vy=0;rec->next=0;
	//new rec initialised to 0
	draw(pt,imageSVG,t,x,y,xx,yy,tre,frmax,frmin,rec);
	for (i=0;i<100;i++)
	{
		j=0;
		while (!(compute_vector(rec)!=0 || j<100))
		{j++;}
		red_vector(rec);
	}
	// while (compute_vector(rec)==0)
	// {}
	
	draw_end(rec,imageSVG,100);
	free(rec);
	
}

PhyloTree *GenerationPhyloTreeBioNJ(point_texte* pt,double *distance)
{
//Algorithme Bio Neighbor-joining
	PhyloTree *tre=PhyloTree_Create();
	PhyloNode *t=0,*t1=0,*t2=0,*t3=0;
	Arbre_Dico* dic=Arbre_Dico_Init();
	Arbre_Dico* di=0;
	int i=pt->nbgarde;
	char* tree1=malloc(str_buffer*(i+1));//laisse un gros buffer de string
	char* strtmp=malloc(str_buffer*2);
	char* strtmp2=malloc(str_buffer*2);
	char* strtmp1=malloc(str_buffer*2);
	strtmp[0]=0;
	strtmp1[0]=0;
	strtmp2[0]=0;
	if (tree1==0)
	{
		printf("erreur malloc tree1\n");
		exit(1);
	}
	char *tree(int i)
	{
		return (tree1+str_buffer*i);
	}
	int j=0,k=0,l=0;
	for (j=1;j<=i;j++) //copie la liste de mots dans tree()
	{
		strcpy(tree(j),mot(j-1));
		t=PhyloNode_Create();
		t->refnumber=j;
		strcpy(t->name,tree(j));
		di=Arbre_Dico_ADD(dic,tree(j));
		di->number=(long int)t;
	}
	double *ad=malloc((i+2)*(i+2)*sizeof(double));
	int *aflag=malloc((i+2)*sizeof(int));
	double *ad1=malloc((i+2)*(i+2)*sizeof(double));
	double *ad2=malloc((i+2)*sizeof(double));
	#define ad(k,l) (*(ad+k*i+l))
	#define ad1(k,l) (*(ad1+k*i+l))
	#define true 1
	#define false 0
	//Initialisation mmoire  0
	for (j=0;j<=i;j++)
	{
		aflag[j]=0;
		ad2[j]=0.0;
		for (k=0;k<=i;k++)
		{
			ad(j,k)=0.0;
			ad1(j,k)=0.0;
		}
	}
	int i1;
	for(i1=1;i1<=i;i1++)
		aflag[i1]=true;
	int j1,j2;
	for(j1=1;j1<=i;j1++)
	{
		ad(j1,j1)=0.0;
		for(j2=1;j2<=i;j2++)
		{
			ad(j1,j2)=distance((j1-1),(j2-1));
			ad1(j1,j2)=ad(j1,j2);
		}
	}
	int k1,k2;
	for(k1=1;k1<=i;k1++)
	{
		for(k2=1;k2<=i;k2++)
			ad2[k1]+=ad(k1,k2);
	}
	int l1;
	for(l1=i;l1>3;l1--)
	{
		double d6 = 1.7976931348623157E+308;
		int i3,j3;
		for(i3=1;i3<i;i3++)
		{
			if(!aflag[i3])
				continue;
			for(j3=i3+1;j3<=i;j3++)
			{
				if(!aflag[j3])
					continue;
				double d7=((double)l1-2.0)*ad(i3,j3)-ad2[i3]-ad2[j3];
				if(d7<d6)
				{
					d6=d7;
					k=i3;
					l=j3;
				}
			}
		}
		double d2=0.5*((ad(k,l)+ad2[k]/((double)l1-2.0))-ad2[l]/((double)l1-2.0));
		double d4=ad(k,l)-d2;
		aflag[l]=false;
		
		strcpy(strtmp,tree(k));
		strcpy(strtmp2,tree(l));
		sprintf(strtmp1,"(%s,%s)",tree(k),tree(l));
		tree(l)[0]=0;
		strcpy(tree(k),strtmp1);
		
		ad2[k]=0.0;
		ad2[l]=0.0;
		double d8=ad1(k,l);
		double d5=0.0;
		if(d8==0.0||l1==3.0)
		{
			d5=0.5;
		} else
		{
			int k3;
			for(k3=1;k3<=i;k3++)
				if(k!=k3&&l!=k3&&ad(0,k3)!=0.0)
					d5+=ad1(k,k3)-ad1(l,k3);
			d5=0.5+d5/(2.0*(double)(l1-2.0)*d8);
			if(d5<0.0)
				d5=0.0;
			if(d5>1.0)
				d5=1.0;
		}
		int l3;
		for(l3=1;l3<=i;l3++)
		{
			if(l3==k||!aflag[l3])
				continue;
			double d=(1.0-d5)*(ad(l3,k)-d2)+d5*(ad(l3,l)-d4);
			if(l3!=k)
				ad2[l3]=(ad2[l3]-ad(l3,k)-ad(l3,l))+d;
			ad2[k]+=d;
			ad(k,l3)=ad(l3,k)=d;
			ad1(k,l3)=((1.0-d5)*ad1(k,l3)+d5*ad1(l,l3))-d5*(1.0-d5)*d8;
			ad1(l3,k)=ad1(k,l3);
		}
		int i4;
		for(i4=1;i4<=i;i4++)
		{
			ad(k,i4)=ad(i4,k);
			ad(i4,l)=0.0;
			ad(l,i4)=0.0;
		}
		
		t=PhyloNode_Create();
		di=Arbre_Dico_ADD(dic,tree(k));
		di->number=(long int)t;
		strcpy(t->name,tree(k));
		di=Arbre_Dico_FOUND(dic,strtmp);
		if (di==0)
		{
			printf("Dico Error %s 1\n",strtmp);return 0;
		}
		t1=(PhyloNode *)di->number;
		t->Fils0=t1;
		t1->weight=d2;
		di=Arbre_Dico_FOUND(dic,strtmp2);
		if (di==0)
		{
			printf("Dico Error%s 2\n",strtmp2);return 0;
		}
		t2=(PhyloNode *)di->number;
		t->Fils1=t2;
		t2->weight=d4;
	}
	int l2;
	for(l2=1;!aflag[l2];l2++);
	k=l2;
	for(l2++;!aflag[l2];l2++);
	l=l2;
	for(l2++;!aflag[l2];l2++);
	int i2=l2;
	
	sprintf(strtmp,"(%s,%s)",tree(l),tree(i2));
	t=PhyloNode_Create();
	di=Arbre_Dico_ADD(dic,strtmp);
	di->number=(long int)t;
	strcpy(t->name,strtmp);

	di=Arbre_Dico_FOUND(dic,tree(l));
	t2=(PhyloNode *)di->number;
	t->Fils0=t2;
	t2->weight=0.5*((ad(k,l)+ad(l,i2))-ad(k,i2));
	di=Arbre_Dico_FOUND(dic,tree(i2));
	t3=(PhyloNode *)di->number;
	t->Fils1=t3;
	t3->weight=0.5*((ad(k,i2)+ad(l,i2))-ad(k,l));
	
	sprintf(strtmp,"(,(%s,%s))",tree(l),tree(i2));
	t1=PhyloNode_Create();
	di=Arbre_Dico_ADD(dic,strtmp);
	di->number=(long int)t1;
	strcpy(t1->name,strtmp);
	t1->Fils0=t;
	t1->Fils1=0;
	
	sprintf(strtmp,"(%s,(,(%s,%s)))",tree(k),tree(l),tree(i2));
	t=PhyloNode_Create();
	di=Arbre_Dico_ADD(dic,strtmp);
	di->number=(long int)t;
	strcpy(t->name,strtmp);
	di=Arbre_Dico_FOUND(dic,tree(k));
	t3=(PhyloNode *)di->number;
	t->Fils0=t3;
	t3->weight=0.5*((ad(k,l)+ad(k,i2))-ad(l,i2));
	t->Fils1=t1;
	tre->root=t;
	strcpy(tree(0),strtmp);
	strcpy(tre->Newick,strtmp);
	return tre;
}

void GenererDessin(point_texte* pt,PhyloTree *tre,int xmax,int ymax,int xx,int yy,char *file,int verbose)
{	
	//Compute the angles on the edges of the tree
	sortLeaves(pt,tre->root,0,tre->angle);
	if (verbose==1)
		printf("Generation sortLeaves termine.\n");
	//Precompute the drawing
	computeDrawing(tre->root,50000,50000,tre);
	if (verbose==1)
		printf("Generation computeDrawing termine.\n");
	text_buffer *imageSVG=malloc(sizeof(text_buffer));
	imageSVG->buffer=malloc(max_buffer);
	imageSVG->size=0;
	imageSVG->size=SVG_Create_Dim(imageSVG->buffer,"Graphe - http://poulphunter1.free.fr/ArbreArbore/",xmax,ymax);
	//Compute the drawing using the width and height computed by the precomputing to scale it properly
	pre_draw(pt,imageSVG,tre->root,xmax,ymax,xx,yy,tre);
	imageSVG->size+=SVG_End(&imageSVG->buffer[imageSVG->size]);
	if (verbose==1)
		printf("Generation dessin termine.\n");
	Write_File(file,imageSVG);
}
